@@ -3,7 +3,6 @@
 '''
 Sphinx documentation for Salt
 '''
-import functools
 import sys
 import os
 import types
@@ -12,7 +11,6 @@ import time
 from sphinx.directives import TocTree
 
 
-# pylint: disable=R0903
 class Mock(object):
     '''
     Mock out specified imports.
@@ -24,7 +22,7 @@ class Mock(object):
 
     http://read-the-docs.readthedocs.org/en/latest/faq.html#i-get-import-errors-on-libraries-that-depend-on-c-modules
     '''
-    def __init__(self, mapping=None, *args, **kwargs):
+    def __init__(self, mapping=None, *args, **kwargs):  # pylint: disable=unused-argument
         """
         Mapping allows autodoc to bypass the Mock object, but actually assign
         a specific value, expected by a specific attribute returned.
@@ -40,12 +38,12 @@ class Mock(object):
         return Mock(mapping=self.__mapping)
 
     def __getattr__(self, name):
-        #__mapping = {'total': 0}
-        data = None
         if name in self.__mapping:
             data = self.__mapping.get(name)
         elif name in ('__file__', '__path__'):
             data = '/dev/null'
+        elif name in ('__mro_entries__', '__qualname__'):
+            raise AttributeError("'Mock' object has no attribute '%s'" % (name))
         else:
             data = Mock(mapping=self.__mapping)
         return data
@@ -53,16 +51,37 @@ class Mock(object):
     def __iter__(self):
         return self
 
-    def next(self):
+    @staticmethod
+    def __next__():
         raise StopIteration
 
-# pylint: enable=R0903
+    # For Python 2
+    next = __next__
+
+
+def mock_decorator_with_params(*oargs, **okwargs):  # pylint: disable=unused-argument
+    '''
+    Optionally mock a decorator that takes parameters
+
+    E.g.:
+
+    @blah(stuff=True)
+    def things():
+        pass
+    '''
+    def inner(fn, *iargs, **ikwargs):  # pylint: disable=unused-argument
+        if hasattr(fn, '__call__'):
+            return fn
+        return Mock()
+    return inner
+
 
 MOCK_MODULES = [
     # Python stdlib
     'user',
 
     # salt core
+    'concurrent',
     'Crypto',
     'Crypto.Signature',
     'Crypto.Cipher',
@@ -104,6 +123,7 @@ MOCK_MODULES = [
 
     'tornado',
     'tornado.concurrent',
+    'tornado.escape',
     'tornado.gen',
     'tornado.httpclient',
     'tornado.httpserver',
@@ -135,8 +155,8 @@ MOCK_MODULES = [
     'pymongo',
     'rabbitmq_server',
     'redis',
-    'requests',
-    'requests.exceptions',
+    #'requests',
+    #'requests.exceptions',
     'rpm',
     'rpmUtils',
     'rpmUtils.arch',
@@ -149,7 +169,6 @@ MOCK_MODULES = [
     'napalm',
     'dson',
     'jnpr',
-    'json',
     'lxml',
     'lxml.etree',
     'jnpr.junos',
@@ -158,6 +177,7 @@ MOCK_MODULES = [
     'jnpr.junos.utils.sw',
     'dns',
     'dns.resolver',
+    'keyring',
     'netaddr',
     'netaddr.IPAddress',
     'netaddr.core',
@@ -173,38 +193,24 @@ MOCK_MODULES = [
     'msgpack',
 ]
 
+MOCK_MODULES_MAPPING = {
+    'cherrypy': {'config': mock_decorator_with_params},
+    'ntsecuritycon': {
+        'STANDARD_RIGHTS_REQUIRED': 0,
+        'SYNCHRONIZE': 0,
+    },
+    'psutil': {'total': 0},  # Otherwise it will crash Sphinx
+}
+
 for mod_name in MOCK_MODULES:
-    if mod_name == 'psutil':
-        mock = Mock(mapping={'total': 0})  # Otherwise it will crash Sphinx
-    else:
-        mock = Mock()
-    sys.modules[mod_name] = mock
-
-def mock_decorator_with_params(*oargs, **okwargs):
-    '''
-    Optionally mock a decorator that takes parameters
-
-    E.g.:
-
-    @blah(stuff=True)
-    def things():
-        pass
-    '''
-    def inner(fn, *iargs, **ikwargs):
-        if hasattr(fn, '__call__'):
-            return fn
-        else:
-            return Mock()
-    return inner
+    sys.modules[mod_name] = Mock(mapping=MOCK_MODULES_MAPPING.get(mod_name))
 
 # Define a fake version attribute for the following libs.
 sys.modules['libcloud'].__version__ = '0.0.0'
+sys.modules['msgpack'].version = (1, 0, 0)
+sys.modules['psutil'].version_info = (3, 0, 0)
 sys.modules['pymongo'].version = '0.0.0'
-sys.modules['ntsecuritycon'].STANDARD_RIGHTS_REQUIRED = 0
-sys.modules['ntsecuritycon'].SYNCHRONIZE = 0
-
-# Define a fake version attribute for the following libs.
-sys.modules['cherrypy'].config = mock_decorator_with_params
+sys.modules['tornado'].version_info = (0, 0, 0)
 
 
 # -- Add paths to PYTHONPATH ---------------------------------------------------
@@ -216,12 +222,12 @@ except NameError:
     docs_basepath = os.path.abspath(os.path.dirname('.'))
 
 addtl_paths = (
-        os.pardir,  # salt itself (for autodoc)
-        '_ext',  # custom Sphinx extensions
+    os.pardir,  # salt itself (for autodoc)
+    '_ext',  # custom Sphinx extensions
 )
 
-for path in addtl_paths:
-    sys.path.insert(0, os.path.abspath(os.path.join(docs_basepath, path)))
+for addtl_path in addtl_paths:
+    sys.path.insert(0, os.path.abspath(os.path.join(docs_basepath, addtl_path)))
 
 
 # We're now able to import salt
@@ -232,8 +238,7 @@ formulas_dir = os.path.join(os.pardir, docs_basepath, 'formulas')
 
 # ----- Intersphinx Settings ------------------------------------------------>
 intersphinx_mapping = {
-        'python2': ('http://docs.python.org/2', None),
-        'python3': ('http://docs.python.org/3', None)
+    'python': ('https://docs.python.org/3', None)
 }
 # <---- Intersphinx Settings -------------------------------------------------
 
@@ -245,9 +250,9 @@ on_saltstack = 'SALT_ON_SALTSTACK' in os.environ
 project = 'Salt'
 
 version = salt.version.__version__
-latest_release = '2017.7.2'  # latest release
-previous_release = '2016.11.8'  # latest release from previous branch
-previous_release_dir = '2016.11'  # path on web server for previous branch
+latest_release = '2018.3.3'  # latest release
+previous_release = '2017.7.8'  # latest release from previous branch
+previous_release_dir = '2017.7'  # path on web server for previous branch
 next_release = ''  # next release
 next_release_dir = ''  # path on web server for next release branch
 
@@ -259,21 +264,21 @@ if on_saltstack:
 
 # < --- START do not merge these settings to other branches START ---> #
 build_type = 'latest'  # latest, previous, develop, next
-release = latest_release # version, latest_release, previous_release
+release = latest_release  # version, latest_release, previous_release
 # < --- END do not merge these settings to other branches END ---> #
 
 # Set google custom search engine
 
 if release == latest_release:
-    search_cx = '004624818632696854117:yfmprrbw3pk' # latest
-elif release.startswith('2014.7'):
-    search_cx = '004624818632696854117:thhslradbru' # 2014.7
-elif release.startswith('2015.5'):
-    search_cx = '004624818632696854117:ovogwef29do' # 2015.5
-elif release.startswith('2015.8'):
-    search_cx = '004624818632696854117:aw_tegffouy' # 2015.8
+    search_cx = '011515552685726825874:ht0p8miksrm' # latest
+elif release.startswith('2018.3'):
+    search_cx = '011515552685726825874:vadptdpvyyu' # 2018.3
+elif release.startswith('2017.7'):
+    search_cx = '011515552685726825874:w-hxmnbcpou' # 2017.7
+elif release.startswith('2016.11'):
+    search_cx = '011515552685726825874:dlsj745pvhq' # 2016.11
 else:
-    search_cx = '004624818632696854117:haj7bjntf4s'  # develop
+    search_cx = '011515552685726825874:x17j5zl74g8'  # develop
 
 needs_sphinx = '1.3'
 
@@ -296,12 +301,12 @@ extensions = [
     'sphinx.ext.intersphinx',
     'httpdomain',
     'youtube',
-    'saltautodoc', # Must be AFTER autodoc
-    'shorturls',
+    #'saltautodoc', # Must be AFTER autodoc
+    #'shorturls',
 ]
 
 try:
-    import sphinxcontrib.spelling
+    import sphinxcontrib.spelling  # false positive, pylint: disable=unused-import
 except ImportError:
     pass
 else:
@@ -318,6 +323,7 @@ rst_prolog = """\
 .. _`salt-users`: https://groups.google.com/forum/#!forum/salt-users
 .. _`salt-announce`: https://groups.google.com/forum/#!forum/salt-announce
 .. _`salt-packagers`: https://groups.google.com/forum/#!forum/salt-packagers
+.. _`salt-slack`: https://saltstackcommunity.herokuapp.com/
 .. |windownload| raw:: html
 
      <p>Python2 x86: <a
@@ -336,19 +342,23 @@ rst_prolog = """\
       | <a href="https://repo.saltstack.com/windows/Salt-Minion-{release}-Py3-AMD64-Setup.exe.md5"><strong>md5</strong></a></p>
 
 
-.. |osxdownload| raw:: html
+.. |osxdownloadpy2| raw:: html
 
-     <p>x86_64: <a href="https://repo.saltstack.com/osx/salt-{release}-x86_64.pkg"><strong>salt-{release}-x86_64.pkg</strong></a>
-      | <a href="https://repo.saltstack.com/osx/salt-{release}-x86_64.pkg.md5"><strong>md5</strong></a></p>
+     <p>x86_64: <a href="https://repo.saltstack.com/osx/salt-{release}-py2-x86_64.pkg"><strong>salt-{release}-py2-x86_64.pkg</strong></a>
+      | <a href="https://repo.saltstack.com/osx/salt-{release}-py2-x86_64.pkg.md5"><strong>md5</strong></a></p>
+
+.. |osxdownloadpy3| raw:: html
+
+     <p>x86_64: <a href="https://repo.saltstack.com/osx/salt-{release}-py3-x86_64.pkg"><strong>salt-{release}-py3-x86_64.pkg</strong></a>
+      | <a href="https://repo.saltstack.com/osx/salt-{release}-py3-x86_64.pkg.md5"><strong>md5</strong></a></p>
 
 """.format(release=release)
 
 # A shortcut for linking to tickets on the GitHub issue tracker
 extlinks = {
     'blob': ('https://github.com/saltstack/salt/blob/%s/%%s' % 'develop', None),
-    'download': ('https://cloud.github.com/downloads/saltstack/salt/%s', None),
-    'issue': ('https://github.com/saltstack/salt/issues/%s', 'issue '),
-    'pull': ('https://github.com/saltstack/salt/pull/%s', 'PR '),
+    'issue': ('https://github.com/saltstack/salt/issues/%s', 'issue #'),
+    'pull': ('https://github.com/saltstack/salt/pull/%s', 'PR #'),
     'formula_url': ('https://github.com/saltstack-formulas/%s', ''),
 }
 
@@ -360,7 +370,8 @@ gettext_compact = False
 
 
 ### HTML options
-html_theme = 'saltstack2' #change to 'saltstack' to use previous theme
+# set 'HTML_THEME=saltstack' to use previous theme
+html_theme = os.environ.get('HTML_THEME', 'saltstack2')
 html_theme_path = ['_themes']
 html_title = u''
 html_short_title = 'Salt'
@@ -428,7 +439,7 @@ html_show_copyright = True
 ### Latex options
 
 latex_documents = [
-  ('contents', 'Salt.tex', 'Salt Documentation', 'SaltStack, Inc.', 'manual'),
+    ('contents', 'Salt.tex', 'Salt Documentation', 'SaltStack, Inc.', 'manual'),
 ]
 
 latex_logo = '_static/salt-logo.png'
@@ -436,7 +447,7 @@ latex_logo = '_static/salt-logo.png'
 latex_elements = {
     'inputenc': '',     # use XeTeX instead of the inputenc LaTeX package.
     'utf8extra': '',
-    'preamble': '''
+    'preamble': r'''
     \usepackage{fontspec}
     \setsansfont{Linux Biolinum O}
     \setromanfont{Linux Libertine O}
@@ -448,35 +459,36 @@ latex_elements = {
 
 
 ### Linkcheck options
-linkcheck_ignore = [r'http://127.0.0.1',
-                    r'http://salt:\d+',
-                    r'http://local:\d+',
-                    r'https://console.aws.amazon.com',
-                    r'http://192.168.33.10',
-                    r'http://domain:\d+',
-                    r'http://123.456.789.012:\d+',
-                    r'http://localhost',
-                    r'https://groups.google.com/forum/#!forum/salt-users',
-                    r'http://logstash.net/docs/latest/inputs/udp',
-                    r'http://logstash.net/docs/latest/inputs/zeromq',
-                    r'http://www.youtube.com/saltstack',
-                    r'https://raven.readthedocs.io',
-                    r'https://getsentry.com',
-                    r'https://salt-cloud.readthedocs.io',
-                    r'https://salt.readthedocs.io',
-                    r'http://www.pip-installer.org/',
-                    r'http://www.windowsazure.com/',
-                    r'https://github.com/watching',
-                    r'dash-feed://',
-                    r'https://github.com/saltstack/salt/',
-                    r'http://bootstrap.saltstack.org',
-                    r'https://bootstrap.saltstack.com',
-                    r'https://raw.githubusercontent.com/saltstack/salt-bootstrap/stable/bootstrap-salt.sh',
-                    r'media.readthedocs.org/dash/salt/latest/salt.xml',
-                    r'https://portal.aws.amazon.com/gp/aws/securityCredentials',
-                    r'https://help.github.com/articles/fork-a-repo',
-                    r'dash-feed://https%3A//media.readthedocs.org/dash/salt/latest/salt.xml'
-                    ]
+linkcheck_ignore = [
+    r'http://127.0.0.1',
+    r'http://salt:\d+',
+    r'http://local:\d+',
+    r'https://console.aws.amazon.com',
+    r'http://192.168.33.10',
+    r'http://domain:\d+',
+    r'http://123.456.789.012:\d+',
+    r'http://localhost',
+    r'https://groups.google.com/forum/#!forum/salt-users',
+    r'http://logstash.net/docs/latest/inputs/udp',
+    r'http://logstash.net/docs/latest/inputs/zeromq',
+    r'http://www.youtube.com/saltstack',
+    r'https://raven.readthedocs.io',
+    r'https://getsentry.com',
+    r'https://salt-cloud.readthedocs.io',
+    r'https://salt.readthedocs.io',
+    r'http://www.pip-installer.org/',
+    r'http://www.windowsazure.com/',
+    r'https://github.com/watching',
+    r'dash-feed://',
+    r'https://github.com/saltstack/salt/',
+    r'http://bootstrap.saltstack.org',
+    r'https://bootstrap.saltstack.com',
+    r'https://raw.githubusercontent.com/saltstack/salt-bootstrap/stable/bootstrap-salt.sh',
+    r'media.readthedocs.org/dash/salt/latest/salt.xml',
+    r'https://portal.aws.amazon.com/gp/aws/securityCredentials',
+    r'https://help.github.com/articles/fork-a-repo',
+    r'dash-feed://https%3A//media.readthedocs.org/dash/salt/latest/salt.xml',
+]
 
 linkcheck_anchors = False
 
@@ -515,10 +527,12 @@ epub_copyright = copyright
 epub_scheme = 'URL'
 epub_identifier = 'http://saltstack.com/'
 
+epub_tocdup = False
 #epub_tocdepth = 3
 
 
 def skip_mod_init_member(app, what, name, obj, skip, options):
+    # pylint: disable=too-many-arguments,unused-argument
     if name.startswith('_'):
         return True
     if isinstance(obj, types.FunctionType) and obj.__name__ == 'mod_init':
