@@ -4,12 +4,12 @@ A simple test engine, not intended for real use but as an example
 '''
 
 # Import python libs
-from __future__ import absolute_import
-import json
+from __future__ import absolute_import, print_function, unicode_literals
 import logging
 
 # Import salt libs
 import salt.utils.event
+import salt.utils.json
 
 log = logging.getLogger(__name__)
 
@@ -34,6 +34,6 @@ def start():
 
     while True:
         event = event_bus.get_event()
-        jevent = json.dumps(event)
+        jevent = salt.utils.json.dumps(event)
         if event:
             log.debug(jevent)
