@@ -4,13 +4,14 @@ Module for fetching artifacts from Artifactory
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import os
 import base64
 import logging
 
 # Import Salt libs
-import salt.utils
+import salt.utils.files
+import salt.utils.stringutils
 import salt.ext.six.moves.http_client  # pylint: disable=import-error,redefined-builtin,no-name-in-module
 from salt.ext.six.moves import urllib  # pylint: disable=no-name-in-module
 from salt.ext.six.moves.urllib.error import HTTPError, URLError  # pylint: disable=no-name-in-module
@@ -38,7 +39,7 @@ def __virtual__():
         return True
 
 
-def get_latest_snapshot(artifactory_url, repository, group_id, artifact_id, packaging, target_dir='/tmp', target_file=None, classifier=None, username=None, password=None):
+def get_latest_snapshot(artifactory_url, repository, group_id, artifact_id, packaging, target_dir='/tmp', target_file=None, classifier=None, username=None, password=None, use_literal_group_id=False):
     '''
        Gets latest snapshot of the given artifact
 
@@ -69,15 +70,15 @@ def get_latest_snapshot(artifactory_url, repository, group_id, artifact_id, pack
     headers = {}
     if username and password:
         headers['Authorization'] = 'Basic {0}'.format(base64.encodestring('{0}:{1}'.format(username, password)).replace('\n', ''))
-    artifact_metadata = _get_artifact_metadata(artifactory_url=artifactory_url, repository=repository, group_id=group_id, artifact_id=artifact_id, headers=headers)
+    artifact_metadata = _get_artifact_metadata(artifactory_url=artifactory_url, repository=repository, group_id=group_id, artifact_id=artifact_id, headers=headers, use_literal_group_id=use_literal_group_id)
     version = artifact_metadata['latest_version']
-    snapshot_url, file_name = _get_snapshot_url(artifactory_url=artifactory_url, repository=repository, group_id=group_id, artifact_id=artifact_id, version=version, packaging=packaging, classifier=classifier, headers=headers)
+    snapshot_url, file_name = _get_snapshot_url(artifactory_url=artifactory_url, repository=repository, group_id=group_id, artifact_id=artifact_id, version=version, packaging=packaging, classifier=classifier, headers=headers, use_literal_group_id=use_literal_group_id)
     target_file = __resolve_target_file(file_name, target_dir, target_file)
 
     return __save_artifact(snapshot_url, target_file, headers)
 
 
-def get_snapshot(artifactory_url, repository, group_id, artifact_id, packaging, version, snapshot_version=None, target_dir='/tmp', target_file=None, classifier=None, username=None, password=None):
+def get_snapshot(artifactory_url, repository, group_id, artifact_id, packaging, version, snapshot_version=None, target_dir='/tmp', target_file=None, classifier=None, username=None, password=None, use_literal_group_id=False):
     '''
        Gets snapshot of the desired version of the artifact
 
@@ -109,13 +110,13 @@ def get_snapshot(artifactory_url, repository, group_id, artifact_id, packaging,
     headers = {}
     if username and password:
         headers['Authorization'] = 'Basic {0}'.format(base64.encodestring('{0}:{1}'.format(username, password)).replace('\n', ''))
-    snapshot_url, file_name = _get_snapshot_url(artifactory_url=artifactory_url, repository=repository, group_id=group_id, artifact_id=artifact_id, version=version, packaging=packaging, snapshot_version=snapshot_version, classifier=classifier, headers=headers)
+    snapshot_url, file_name = _get_snapshot_url(artifactory_url=artifactory_url, repository=repository, group_id=group_id, artifact_id=artifact_id, version=version, packaging=packaging, snapshot_version=snapshot_version, classifier=classifier, headers=headers, use_literal_group_id=use_literal_group_id)
     target_file = __resolve_target_file(file_name, target_dir, target_file)
 
     return __save_artifact(snapshot_url, target_file, headers)
 
 
-def get_latest_release(artifactory_url, repository, group_id, artifact_id, packaging, target_dir='/tmp', target_file=None, classifier=None, username=None, password=None):
+def get_latest_release(artifactory_url, repository, group_id, artifact_id, packaging, target_dir='/tmp', target_file=None, classifier=None, username=None, password=None, use_literal_group_id=False):
     '''
        Gets the latest release of the artifact
 
@@ -146,13 +147,13 @@ def get_latest_release(artifactory_url, repository, group_id, artifact_id, packa
     if username and password:
         headers['Authorization'] = 'Basic {0}'.format(base64.encodestring('{0}:{1}'.format(username, password)).replace('\n', ''))
     version = __find_latest_version(artifactory_url=artifactory_url, repository=repository, group_id=group_id, artifact_id=artifact_id, headers=headers)
-    release_url, file_name = _get_release_url(repository, group_id, artifact_id, packaging, version, artifactory_url, classifier)
+    release_url, file_name = _get_release_url(repository, group_id, artifact_id, packaging, version, artifactory_url, classifier, use_literal_group_id)
     target_file = __resolve_target_file(file_name, target_dir, target_file)
 
     return __save_artifact(release_url, target_file, headers)
 
 
-def get_release(artifactory_url, repository, group_id, artifact_id, packaging, version, target_dir='/tmp', target_file=None, classifier=None, username=None, password=None):
+def get_release(artifactory_url, repository, group_id, artifact_id, packaging, version, target_dir='/tmp', target_file=None, classifier=None, username=None, password=None, use_literal_group_id=False):
     '''
        Gets the specified release of the artifact
 
@@ -184,7 +185,7 @@ def get_release(artifactory_url, repository, group_id, artifact_id, packaging, v
     headers = {}
     if username and password:
         headers['Authorization'] = 'Basic {0}'.format(base64.encodestring('{0}:{1}'.format(username, password)).replace('\n', ''))
-    release_url, file_name = _get_release_url(repository, group_id, artifact_id, packaging, version, artifactory_url, classifier)
+    release_url, file_name = _get_release_url(repository, group_id, artifact_id, packaging, version, artifactory_url, classifier, use_literal_group_id)
     target_file = __resolve_target_file(file_name, target_dir, target_file)
 
     return __save_artifact(release_url, target_file, headers)
@@ -196,7 +197,7 @@ def __resolve_target_file(file_name, target_dir, target_file=None):
     return target_file
 
 
-def _get_snapshot_url(artifactory_url, repository, group_id, artifact_id, version, packaging, snapshot_version=None, classifier=None, headers=None):
+def _get_snapshot_url(artifactory_url, repository, group_id, artifact_id, version, packaging, snapshot_version=None, classifier=None, headers=None, use_literal_group_id=False):
     if headers is None:
         headers = {}
     has_classifier = classifier is not None and classifier != ""
@@ -245,7 +246,7 @@ def _get_snapshot_url(artifactory_url, repository, group_id, artifact_id, versio
             log.error('Could not fetch maven-metadata.xml. Assuming snapshot_version=%s.', version)
             snapshot_version = version
 
-    group_url = __get_group_id_subpath(group_id)
+    group_url = __get_group_id_subpath(group_id, use_literal_group_id)
 
     file_name = '{artifact_id}-{snapshot_version}{classifier}.{packaging}'.format(
         artifact_id=artifact_id,
@@ -265,8 +266,8 @@ def _get_snapshot_url(artifactory_url, repository, group_id, artifact_id, versio
     return snapshot_url, file_name
 
 
-def _get_release_url(repository, group_id, artifact_id, packaging, version, artifactory_url, classifier=None):
-    group_url = __get_group_id_subpath(group_id)
+def _get_release_url(repository, group_id, artifact_id, packaging, version, artifactory_url, classifier=None, use_literal_group_id=False):
+    group_url = __get_group_id_subpath(group_id, use_literal_group_id)
 
     # for released versions the suffix for the file is same as version
     file_name = '{artifact_id}-{version}{classifier}.{packaging}'.format(
@@ -286,8 +287,8 @@ def _get_release_url(repository, group_id, artifact_id, packaging, version, arti
     return release_url, file_name
 
 
-def _get_artifact_metadata_url(artifactory_url, repository, group_id, artifact_id):
-    group_url = __get_group_id_subpath(group_id)
+def _get_artifact_metadata_url(artifactory_url, repository, group_id, artifact_id, use_literal_group_id=False):
+    group_url = __get_group_id_subpath(group_id, use_literal_group_id)
     # for released versions the suffix for the file is same as version
     artifact_metadata_url = '{artifactory_url}/{repository}/{group_url}/{artifact_id}/maven-metadata.xml'.format(
                                  artifactory_url=artifactory_url,
@@ -298,13 +299,14 @@ def _get_artifact_metadata_url(artifactory_url, repository, group_id, artifact_i
     return artifact_metadata_url
 
 
-def _get_artifact_metadata_xml(artifactory_url, repository, group_id, artifact_id, headers):
+def _get_artifact_metadata_xml(artifactory_url, repository, group_id, artifact_id, headers, use_literal_group_id=False):
 
     artifact_metadata_url = _get_artifact_metadata_url(
         artifactory_url=artifactory_url,
         repository=repository,
         group_id=group_id,
-        artifact_id=artifact_id
+        artifact_id=artifact_id,
+        use_literal_group_id=use_literal_group_id
     )
 
     try:
@@ -321,8 +323,8 @@ def _get_artifact_metadata_xml(artifactory_url, repository, group_id, artifact_i
     return artifact_metadata_xml
 
 
-def _get_artifact_metadata(artifactory_url, repository, group_id, artifact_id, headers):
-    metadata_xml = _get_artifact_metadata_xml(artifactory_url=artifactory_url, repository=repository, group_id=group_id, artifact_id=artifact_id, headers=headers)
+def _get_artifact_metadata(artifactory_url, repository, group_id, artifact_id, headers, use_literal_group_id=False):
+    metadata_xml = _get_artifact_metadata_xml(artifactory_url=artifactory_url, repository=repository, group_id=group_id, artifact_id=artifact_id, headers=headers, use_literal_group_id=use_literal_group_id)
     root = ET.fromstring(metadata_xml)
 
     assert group_id == root.find('groupId').text
@@ -334,8 +336,8 @@ def _get_artifact_metadata(artifactory_url, repository, group_id, artifact_id, h
 
 
 # functions for handling snapshots
-def _get_snapshot_version_metadata_url(artifactory_url, repository, group_id, artifact_id, version):
-    group_url = __get_group_id_subpath(group_id)
+def _get_snapshot_version_metadata_url(artifactory_url, repository, group_id, artifact_id, version, use_literal_group_id=False):
+    group_url = __get_group_id_subpath(group_id, use_literal_group_id)
     # for released versions the suffix for the file is same as version
     snapshot_version_metadata_url = '{artifactory_url}/{repository}/{group_url}/{artifact_id}/{version}/maven-metadata.xml'.format(
                                          artifactory_url=artifactory_url,
@@ -347,14 +349,15 @@ def _get_snapshot_version_metadata_url(artifactory_url, repository, group_id, ar
     return snapshot_version_metadata_url
 
 
-def _get_snapshot_version_metadata_xml(artifactory_url, repository, group_id, artifact_id, version, headers):
+def _get_snapshot_version_metadata_xml(artifactory_url, repository, group_id, artifact_id, version, headers, use_literal_group_id=False):
 
     snapshot_version_metadata_url = _get_snapshot_version_metadata_url(
         artifactory_url=artifactory_url,
         repository=repository,
         group_id=group_id,
         artifact_id=artifact_id,
-        version=version
+        version=version,
+        use_literal_group_id=use_literal_group_id
     )
 
     try:
@@ -391,8 +394,8 @@ def _get_snapshot_version_metadata(artifactory_url, repository, group_id, artifa
     }
 
 
-def __get_latest_version_url(artifactory_url, repository, group_id, artifact_id):
-    group_url = __get_group_id_subpath(group_id)
+def __get_latest_version_url(artifactory_url, repository, group_id, artifact_id, use_literal_group_id=False):
+    group_url = __get_group_id_subpath(group_id, use_literal_group_id)
     # for released versions the suffix for the file is same as version
     latest_version_url = '{artifactory_url}/api/search/latestVersion?g={group_url}&a={artifact_id}&repos={repository}'.format(
                                  artifactory_url=artifactory_url,
@@ -403,13 +406,14 @@ def __get_latest_version_url(artifactory_url, repository, group_id, artifact_id)
     return latest_version_url
 
 
-def __find_latest_version(artifactory_url, repository, group_id, artifact_id, headers):
+def __find_latest_version(artifactory_url, repository, group_id, artifact_id, headers, use_literal_group_id=False):
 
     latest_version_url = __get_latest_version_url(
         artifactory_url=artifactory_url,
         repository=repository,
         group_id=group_id,
-        artifact_id=artifact_id
+        artifact_id=artifact_id,
+        use_literal_group_id=use_literal_group_id
     )
 
     try:
@@ -439,7 +443,7 @@ def __save_artifact(artifact_url, target_file, headers):
     }
 
     if os.path.isfile(target_file):
-        log.debug("File {0} already exists, checking checksum...".format(target_file))
+        log.debug("File %s already exists, checking checksum...", target_file)
         checksum_url = artifact_url + ".sha1"
 
         checksum_success, artifact_sum, checksum_comment = __download(checksum_url, headers)
@@ -463,13 +467,13 @@ def __save_artifact(artifact_url, target_file, headers):
             result['comment'] = checksum_comment
             return result
 
-    log.debug('Downloading: {url} -> {target_file}'.format(url=artifact_url, target_file=target_file))
+    log.debug('Downloading: %s -> %s', artifact_url, target_file)
 
     try:
         request = urllib.request.Request(artifact_url, None, headers)
         f = urllib.request.urlopen(request)
-        with salt.utils.fopen(target_file, "wb") as local_file:
-            local_file.write(f.read())
+        with salt.utils.files.fopen(target_file, "wb") as local_file:
+            local_file.write(salt.utils.stringutils.to_bytes(f.read()))
         result['status'] = True
         result['comment'] = __append_comment(('Artifact downloaded from URL: {0}'.format(artifact_url)), result['comment'])
         result['changes']['downloaded_file'] = target_file
@@ -481,9 +485,11 @@ def __save_artifact(artifact_url, target_file, headers):
     return result
 
 
-def __get_group_id_subpath(group_id):
-    group_url = group_id.replace('.', '/')
-    return group_url
+def __get_group_id_subpath(group_id, use_literal_group_id=False):
+    if not use_literal_group_id:
+        group_url = group_id.replace('.', '/')
+        return group_url
+    return group_id
 
 
 def __get_classifier_url(classifier):
@@ -492,7 +498,7 @@ def __get_classifier_url(classifier):
 
 
 def __download(request_url, headers):
-    log.debug('Downloading content from {0}'.format(request_url))
+    log.debug('Downloading content from %s', request_url)
 
     success = False
     content = None
