@@ -5,28 +5,27 @@ Support for iptables
 Configuration Options
 ---------------------
 
-The following options can be set in the :ref:`minion config
-<configuration-salt-minion>`, :ref:`minion grains<configuration-minion-grains>`
-, :ref:`minion pillar<configuration-minion-pillar>`, or
-`master config<configuration-salt-master>`.
+The following options can be set in the minion config, grains, pillar, or
+master config. The configuration is read using :py:func:`config.get
+<salt.modules.config.get>`.
 
 - ``iptables.save_filters``: List of REGEX strings to FILTER OUT matching lines
 
-    This is useful for filtering out chains, rules, etc that you do not
-    wish to persist, such as ephemeral Docker rules.
+  This is useful for filtering out chains, rules, etc that you do not wish to
+  persist, such as ephemeral Docker rules.
 
-    The default is to not filter out anything.
+  The default is to not filter out anything.
 
-    .. code-block:: yaml
+  .. code-block:: yaml
 
-        iptables.save_filters:
-           - "-j CATTLE_PREROUTING"
-           - "-j DOCKER"
-           - "-A POSTROUTING"
-           - "-A CATTLE_POSTROUTING"
-           - "-A FORWARD"
+      iptables.save_filters:
+        - "-j CATTLE_PREROUTING"
+        - "-j DOCKER"
+        - "-A POSTROUTING"
+        - "-A CATTLE_POSTROUTING"
+        - "-A FORWARD"
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 
 # Import python libs
 import os
@@ -36,7 +35,9 @@ import uuid
 import string
 
 # Import salt libs
-import salt.utils
+import salt.utils.args
+import salt.utils.files
+import salt.utils.path
 from salt.state import STATE_INTERNAL_KEYWORDS as _STATE_INTERNAL_KEYWORDS
 from salt.exceptions import SaltException
 from salt.ext import six
@@ -49,7 +50,7 @@ def __virtual__():
     '''
     Only load the module if iptables is installed
     '''
-    if not salt.utils.which('iptables'):
+    if not salt.utils.path.which('iptables'):
         return (False, 'The iptables execution module cannot be loaded: iptables not installed.')
 
     return True
@@ -60,9 +61,9 @@ def _iptables_cmd(family='ipv4'):
     Return correct command based on the family, e.g. ipv4 or ipv6
     '''
     if family == 'ipv6':
-        return salt.utils.which('ip6tables')
+        return salt.utils.path.which('ip6tables')
     else:
-        return salt.utils.which('iptables')
+        return salt.utils.path.which('iptables')
 
 
 def _has_option(option, family='ipv4'):
@@ -107,6 +108,11 @@ def _conf(family='ipv4'):
     elif __grains__['os_family'] == 'Suse':
         # SuSE does not seem to use separate files for IPv4 and IPv6
         return '/etc/sysconfig/scripts/SuSEfirewall2-custom'
+    elif __grains__['os_family'] == 'Void':
+        if family == 'ipv6':
+            return '/etc/iptables/iptables.rules'
+        else:
+            return '/etc/iptables/ip6tables.rules'
     elif __grains__['os'] == 'Alpine':
         if family == 'ipv6':
             return '/etc/iptables/rules6-save'
@@ -153,8 +159,8 @@ def _regex_iptables_save(cmd_output, filters=None):
                 __context__['iptables.save_filters']\
                     .append(re.compile(pattern))
             except re.error as e:
-                log.warning('Skipping regex rule: \'{0}\': {1}'
-                            .format(pattern, e))
+                log.warning('Skipping regex rule: \'%s\': %s',
+                            pattern, e)
                 continue
 
     if len(__context__['iptables.save_filters']) > 0:
@@ -200,11 +206,20 @@ def build_rule(table='filter', chain=None, command=None, position='', full=None,
     If a position is required (as with `-I` or `-D`), it may be specified as
     `position`. This will only be useful if `full` is True.
 
+    If `state` is passed, it will be ignored, use `connstate`.
     If `connstate` is passed in, it will automatically be changed to `state`.
 
     To pass in jump options that doesn't take arguments, pass in an empty
     string.
 
+    .. note::
+
+        Whereas iptables will accept ``-p``, ``--proto[c[o[l]]]`` as synonyms
+        of ``--protocol``, if ``--proto`` appears in an iptables command after
+        the appearance of ``-m policy``, it is interpreted as the ``--proto``
+        option of the policy extension (see the iptables-extensions(8) man
+        page).
+
     CLI Examples:
 
     .. code-block:: bash
@@ -213,19 +228,19 @@ def build_rule(table='filter', chain=None, command=None, position='', full=None,
             connstate=RELATED,ESTABLISHED jump=ACCEPT
 
         salt '*' iptables.build_rule filter INPUT command=I position=3 \\
-            full=True match=state state=RELATED,ESTABLISHED jump=ACCEPT
+            full=True match=state connstate=RELATED,ESTABLISHED jump=ACCEPT
 
         salt '*' iptables.build_rule filter INPUT command=A \\
-            full=True match=state state=RELATED,ESTABLISHED \\
+            full=True match=state connstate=RELATED,ESTABLISHED \\
             source='127.0.0.1' jump=ACCEPT
 
         .. Invert Rules
         salt '*' iptables.build_rule filter INPUT command=A \\
-            full=True match=state state=RELATED,ESTABLISHED \\
-            source='! 127.0.0.1' jump=ACCEPT
+            full=True match=state connstate=RELATED,ESTABLISHED \\
+            source='!127.0.0.1' jump=ACCEPT
 
         salt '*' iptables.build_rule filter INPUT command=A \\
-            full=True match=state state=RELATED,ESTABLISHED \\
+            full=True match=state connstate=RELATED,ESTABLISHED \\
             destination='not 127.0.0.1' jump=ACCEPT
 
         IPv6:
@@ -233,9 +248,8 @@ def build_rule(table='filter', chain=None, command=None, position='', full=None,
             connstate=RELATED,ESTABLISHED jump=ACCEPT \\
             family=ipv6
         salt '*' iptables.build_rule filter INPUT command=I position=3 \\
-            full=True match=state state=RELATED,ESTABLISHED jump=ACCEPT \\
+            full=True match=state connstate=RELATED,ESTABLISHED jump=ACCEPT \\
             family=ipv6
-
     '''
     if 'target' in kwargs:
         kwargs['jump'] = kwargs.pop('target')
@@ -249,7 +263,7 @@ def build_rule(table='filter', chain=None, command=None, position='', full=None,
             del kwargs[ignore]
 
     rule = []
-    proto = False
+    protocol = False
     bang_not_pat = re.compile(r'(!|not)\s?')
 
     def maybe_add_negation(arg):
@@ -273,12 +287,15 @@ def build_rule(table='filter', chain=None, command=None, position='', full=None,
         rule.append('{0}-o {1}'.format(maybe_add_negation('of'), kwargs['of']))
         del kwargs['of']
 
-    for proto_arg in ('protocol', 'proto'):
-        if proto_arg in kwargs:
-            if not proto:
-                rule.append('{0}-p {1}'.format(maybe_add_negation(proto_arg), kwargs[proto_arg]))
-                proto = True
-            del kwargs[proto_arg]
+    if 'proto' in kwargs and kwargs.get('match') != 'policy':
+        kwargs['protocol'] = kwargs['proto']
+        del kwargs['proto']
+        # Handle the case 'proto' in kwargs and kwargs.get('match') == 'policy' below
+    if 'protocol' in kwargs:
+        if not protocol:
+            rule.append('{0}-p {1}'.format(maybe_add_negation('protocol'), kwargs['protocol']))
+            protocol = True
+        del kwargs['protocol']
 
     if 'match' in kwargs:
         match_value = kwargs['match']
@@ -289,6 +306,9 @@ def build_rule(table='filter', chain=None, command=None, position='', full=None,
             if 'name_' in kwargs and match.strip() in ('pknock', 'quota2', 'recent'):
                 rule.append('--name {0}'.format(kwargs['name_']))
                 del kwargs['name_']
+        if 'proto' in kwargs and kwargs.get('match') == 'policy':
+            rule.append('{0}--proto {1}'.format(maybe_add_negation('proto'), kwargs['proto']))
+            del kwargs['proto']
         del kwargs['match']
 
     if 'match-set' in kwargs:
@@ -322,8 +342,8 @@ def build_rule(table='filter', chain=None, command=None, position='', full=None,
         if multiport_arg in kwargs:
             if '-m multiport' not in rule:
                 rule.append('-m multiport')
-                if not proto:
-                    return 'Error: proto must be specified'
+                if not protocol:
+                    return 'Error: protocol must be specified'
 
             mp_value = kwargs[multiport_arg]
             if isinstance(mp_value, list):
@@ -504,7 +524,7 @@ def build_rule(table='filter', chain=None, command=None, position='', full=None,
 
     rule += after_jump
 
-    if full in ['True', 'true']:
+    if full:
         if not table:
             return 'Error: Table needs to be specified'
         if not chain:
@@ -650,7 +670,7 @@ def save(filename=None, family='ipv4'):
     if _conf() and not filename:
         filename = _conf(family)
 
-    log.debug('Saving rules to {0}'.format(filename))
+    log.debug('Saving rules to %s', filename)
 
     parent_dir = os.path.dirname(filename)
     if not os.path.isdir(parent_dir):
@@ -967,7 +987,7 @@ def _parse_conf(conf_file=None, in_mem=False, family='ipv4'):
 
     rules = ''
     if conf_file:
-        with salt.utils.fopen(conf_file, 'r') as ifile:
+        with salt.utils.files.fopen(conf_file, 'r') as ifile:
             rules = ifile.read()
     elif in_mem:
         cmd = '{0}-save' . format(_iptables_cmd(family))
@@ -979,6 +999,7 @@ def _parse_conf(conf_file=None, in_mem=False, family='ipv4'):
     table = ''
     parser = _parser()
     for line in rules.splitlines():
+        line = salt.utils.stringutils.to_unicode(line)
         if line.startswith('*'):
             table = line.replace('*', '')
             ret[table] = {}
@@ -994,7 +1015,7 @@ def _parse_conf(conf_file=None, in_mem=False, family='ipv4'):
             ret[table][chain]['rules'] = []
             ret[table][chain]['rules_comment'] = {}
         elif line.startswith('-A'):
-            args = salt.utils.shlex_split(line)
+            args = salt.utils.args.shlex_split(line)
             index = 0
             while index + 1 < len(args):
                 swap = args[index] == '!' and args[index + 1].startswith('-')
@@ -1014,11 +1035,8 @@ def _parse_conf(conf_file=None, in_mem=False, family='ipv4'):
             if args[-1].startswith('-'):
                 args.append('')
             parsed_args = []
-            if sys.version.startswith('2.6'):
-                (opts, leftover_args) = parser.parse_args(args)
-                parsed_args = vars(opts)
-            else:
-                parsed_args = vars(parser.parse_args(args))
+            opts, _ = parser.parse_known_args(args)
+            parsed_args = vars(opts)
             ret_args = {}
             chain = parsed_args['append']
             for arg in parsed_args:
@@ -1033,9 +1051,9 @@ def _parse_conf(conf_file=None, in_mem=False, family='ipv4'):
 
 def _parser():
     '''
-    This function contains _all_ the options I could find in man 8 iptables,
-    listed in the first section that I found them in. They will not all be used
-    by all parts of the module; use them intelligently and appropriately.
+    This function attempts to list all the options documented in the
+    iptables(8) and iptables-extensions(8) man pages.  They will not all be
+    used by all parts of the module; use them intelligently and appropriately.
     '''
     add_arg = None
     if sys.version.startswith('2.6'):
