@@ -10,7 +10,7 @@ Manage users on Mac OS 10.7+
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 try:
     import pwd
 except ImportError:
@@ -23,11 +23,15 @@ from salt.ext.six.moves import range, map  # pylint: disable=import-error,redefi
 from salt.ext.six import string_types
 
 # Import salt libs
-import salt.utils
+import salt.utils.args
+import salt.utils.data
+import salt.utils.decorators.path
 import salt.utils.files
-import salt.utils.decorators as decorators
+import salt.utils.stringutils
+import salt.utils.user
 from salt.utils.locales import sdecode as _sdecode
 from salt.exceptions import CommandExecutionError, SaltInvocationError
+from salt.ext import six
 
 log = logging.getLogger(__name__)
 
@@ -96,7 +100,7 @@ def add(name,
     if info(name):
         raise CommandExecutionError('User \'{0}\' already exists'.format(name))
 
-    if salt.utils.contains_whitespace(name):
+    if salt.utils.stringutils.contains_whitespace(name):
         raise SaltInvocationError('Username cannot contain whitespace')
 
     if uid is None:
@@ -143,7 +147,7 @@ def delete(name, remove=False, force=False):
 
         salt '*' user.delete name remove=True force=True
     '''
-    if salt.utils.contains_whitespace(name):
+    if salt.utils.stringutils.contains_whitespace(name):
         raise SaltInvocationError('Username cannot contain whitespace')
     if not info(name):
         return True
@@ -272,10 +276,10 @@ def chhome(name, home, **kwargs):
 
         salt '*' user.chhome foo /Users/foo
     '''
-    kwargs = salt.utils.clean_kwargs(**kwargs)
+    kwargs = salt.utils.args.clean_kwargs(**kwargs)
     persist = kwargs.pop('persist', False)
     if kwargs:
-        salt.utils.invalid_kwargs(kwargs)
+        salt.utils.args.invalid_kwargs(kwargs)
     if persist:
         log.info('Ignoring unsupported \'persist\' argument to user.chhome')
 
@@ -359,13 +363,13 @@ def chgroups(name, groups, append=False):
     if isinstance(groups, string_types):
         groups = groups.split(',')
 
-    bad_groups = [x for x in groups if salt.utils.contains_whitespace(x)]
+    bad_groups = [x for x in groups if salt.utils.stringutils.contains_whitespace(x)]
     if bad_groups:
         raise SaltInvocationError(
             'Invalid group name(s): {0}'.format(', '.join(bad_groups))
         )
     ugrps = set(list_groups(name))
-    desired = set(str(x) for x in groups if bool(str(x)))
+    desired = set(six.text_type(x) for x in groups if bool(six.text_type(x)))
     primary_group = __salt__['file.gid_to_group'](uinfo['gid'])
     if primary_group:
         desired.add(primary_group)
@@ -419,7 +423,7 @@ def _format_info(data):
             'fullname': data.pw_gecos}
 
 
-@decorators.which('id')
+@salt.utils.decorators.path.which('id')
 def primary_group(name):
     '''
     Return the primary group of the named user
@@ -453,7 +457,7 @@ def list_groups(name):
 
         salt '*' user.list_groups foo
     '''
-    groups = [group for group in salt.utils.get_group_list(name)]
+    groups = [group for group in salt.utils.user.get_group_list(name)]
     return groups
 
 
@@ -467,7 +471,8 @@ def list_users():
 
         salt '*' user.list_users
     '''
-    return sorted([user.pw_name for user in pwd.getpwall()])
+    users = _dscl(['/users'], 'list')['stdout']
+    return users.split()
 
 
 def rename(name, new_name):
@@ -568,7 +573,7 @@ def _kcpassword(password):
 
     # Convert each byte back to a character
     password = list(map(chr, password))
-    return ''.join(password)
+    return b''.join(salt.utils.data.encode(password))
 
 
 def enable_auto_login(name, password):
@@ -606,7 +611,7 @@ def enable_auto_login(name, password):
     # Create/Update the kcpassword file with an obfuscated password
     o_password = _kcpassword(password=password)
     with salt.utils.files.set_umask(0o077):
-        with salt.utils.fopen('/etc/kcpassword', 'w') as fd:
+        with salt.utils.files.fopen('/etc/kcpassword', 'w' if six.PY2 else 'wb') as fd:
             fd.write(o_password)
 
     return current if isinstance(current, bool) else current.lower() == name.lower()
