@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 '''
 NAPALM Network
-===============
+==============
 
 Basic methods for interaction with the network device through the virtual proxy 'napalm'.
 
@@ -19,18 +19,20 @@ Dependencies
 .. versionchanged:: 2017.7.0
 '''
 
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 
-# Import python lib
+# Import Python lib
 import logging
 log = logging.getLogger(__name__)
 
-# salt libs
-from salt.ext import six
-import salt.utils.templates
+# Import Salt libs
+import salt.utils.files
 import salt.utils.napalm
+import salt.utils.templates
 import salt.utils.versions
-from salt.utils.napalm import proxy_napalm_wrap
+
+# Import 3rd-party libs
+from salt.ext import six
 
 # ----------------------------------------------------------------------------------------------------------------------
 # module properties
@@ -99,7 +101,7 @@ def _filter_dict(input_dict, search_key, search_value):
 
 def _explicit_close(napalm_device):
     '''
-    Will explicitely close the config session with the network device,
+    Will explicily close the config session with the network device,
     when running in a now-always-alive proxy minion or regular minion.
     This helper must be used in configuration-related functions,
     as the session is preserved and not closed before making any changes.
@@ -137,7 +139,7 @@ def _config_logic(napalm_device,
     # then the decorator will make sure that
     # if not proxy (when the connection is always alive)
     # and the `inherit_napalm_device` is set,
-    # `napalm_device` will be overriden.
+    # `napalm_device` will be overridden.
     # See `salt.utils.napalm.proxy_napalm_wrap` decorator.
 
     loaded_result['already_configured'] = False
@@ -228,7 +230,7 @@ def _config_logic(napalm_device,
 # ----------------------------------------------------------------------------------------------------------------------
 
 
-@proxy_napalm_wrap
+@salt.utils.napalm.proxy_napalm_wrap
 def connected(**kwargs):  # pylint: disable=unused-argument
     '''
     Specifies if the connection to the device succeeded.
@@ -245,7 +247,7 @@ def connected(**kwargs):  # pylint: disable=unused-argument
     }
 
 
-@proxy_napalm_wrap
+@salt.utils.napalm.proxy_napalm_wrap
 def facts(**kwargs):  # pylint: disable=unused-argument
     '''
     Returns characteristics of the network device.
@@ -271,7 +273,7 @@ def facts(**kwargs):  # pylint: disable=unused-argument
     .. code-block:: python
 
         {
-            'os_version': u'13.3R6.5',
+            'os_version': '13.3R6.5',
             'uptime': 10117140,
             'interface_list': [
                 'lc-0/0/0',
@@ -284,11 +286,11 @@ def facts(**kwargs):  # pylint: disable=unused-argument
                 'gr-0/0/10',
                 'ip-0/0/10'
             ],
-            'vendor': u'Juniper',
-            'serial_number': u'JN131356FBFA',
-            'model': u'MX480',
-            'hostname': u're0.edge05.syd01',
-            'fqdn': u're0.edge05.syd01'
+            'vendor': 'Juniper',
+            'serial_number': 'JN131356FBFA',
+            'model': 'MX480',
+            'hostname': 're0.edge05.syd01',
+            'fqdn': 're0.edge05.syd01'
         }
     '''
 
@@ -300,7 +302,7 @@ def facts(**kwargs):  # pylint: disable=unused-argument
     )
 
 
-@proxy_napalm_wrap
+@salt.utils.napalm.proxy_napalm_wrap
 def environment(**kwargs):  # pylint: disable=unused-argument
     '''
     Returns the environment of the device.
@@ -367,14 +369,129 @@ def environment(**kwargs):  # pylint: disable=unused-argument
     )
 
 
-@proxy_napalm_wrap
+@salt.utils.napalm.proxy_napalm_wrap
 def cli(*commands, **kwargs):  # pylint: disable=unused-argument
-
     '''
     Returns a dictionary with the raw output of all commands passed as arguments.
 
-    :param commands: list of commands to be executed on the device
-    :return: a dictionary with the mapping between each command and its raw output
+    commands
+        List of commands to be executed on the device.
+
+    textfsm_parse: ``False``
+        Try parsing the outputs using the TextFSM templates.
+
+        .. versionadded:: 2018.3.0
+
+        .. note::
+            This option can be also specified in the minion configuration
+            file or pillar as ``napalm_cli_textfsm_parse``.
+
+    textfsm_path
+        The path where the TextFSM templates can be found. This option implies
+        the usage of the TextFSM index file.
+        ``textfsm_path`` can be either absolute path on the server,
+        either specified using the following URL mschemes: ``file://``,
+        ``salt://``, ``http://``, ``https://``, ``ftp://``,
+        ``s3://``, ``swift://``.
+
+        .. versionadded:: 2018.3.0
+
+        .. note::
+            This needs to be a directory with a flat structure, having an
+            index file (whose name can be specified using the ``index_file`` option)
+            and a number of TextFSM templates.
+
+        .. note::
+            This option can be also specified in the minion configuration
+            file or pillar as ``textfsm_path``.
+
+    textfsm_template
+        The path to a certain the TextFSM template.
+        This can be specified using the absolute path
+        to the file, or using one of the following URL schemes:
+
+        - ``salt://``, to fetch the template from the Salt fileserver.
+        - ``http://`` or ``https://``
+        - ``ftp://``
+        - ``s3://``
+        - ``swift://``
+
+        .. versionadded:: 2018.3.0
+
+    textfsm_template_dict
+        A dictionary with the mapping between a command
+        and the corresponding TextFSM path to use to extract the data.
+        The TextFSM paths can be specified as in ``textfsm_template``.
+
+        .. versionadded:: 2018.3.0
+
+        .. note::
+            This option can be also specified in the minion configuration
+            file or pillar as ``napalm_cli_textfsm_template_dict``.
+
+    platform_grain_name: ``os``
+        The name of the grain used to identify the platform name
+        in the TextFSM index file. Default: ``os``.
+
+        .. versionadded:: 2018.3.0
+
+        .. note::
+            This option can be also specified in the minion configuration
+            file or pillar as ``textfsm_platform_grain``.
+
+    platform_column_name: ``Platform``
+        The column name used to identify the platform,
+        exactly as specified in the TextFSM index file.
+        Default: ``Platform``.
+
+        .. versionadded:: 2018.3.0
+
+        .. note::
+            This is field is case sensitive, make sure
+            to assign the correct value to this option,
+            exactly as defined in the index file.
+
+        .. note::
+            This option can be also specified in the minion configuration
+            file or pillar as ``textfsm_platform_column_name``.
+
+    index_file: ``index``
+        The name of the TextFSM index file, under the ``textfsm_path``. Default: ``index``.
+
+        .. versionadded:: 2018.3.0
+
+        .. note::
+            This option can be also specified in the minion configuration
+            file or pillar as ``textfsm_index_file``.
+
+    saltenv: ``base``
+        Salt fileserver envrionment from which to retrieve the file.
+        Ignored if ``textfsm_path`` is not a ``salt://`` URL.
+
+        .. versionadded:: 2018.3.0
+
+    include_empty: ``False``
+        Include empty files under the ``textfsm_path``.
+
+        .. versionadded:: 2018.3.0
+
+    include_pat
+        Glob or regex to narrow down the files cached from the given path.
+        If matching with a regex, the regex must be prefixed with ``E@``,
+        otherwise the expression will be interpreted as a glob.
+
+        .. versionadded:: 2018.3.0
+
+    exclude_pat
+        Glob or regex to exclude certain files from being cached from the given path.
+        If matching with a regex, the regex must be prefixed with ``E@``,
+        otherwise the expression will be interpreted as a glob.
+
+        .. versionadded:: 2018.3.0
+
+        .. note::
+            If used with ``include_pat``, files matching this pattern will be
+            excluded from the subset of files defined by ``include_pat``.
 
     CLI Example:
 
@@ -382,19 +499,25 @@ def cli(*commands, **kwargs):  # pylint: disable=unused-argument
 
         salt '*' net.cli "show version" "show chassis fan"
 
+    CLI Example with TextFSM template:
+
+    .. code-block:: bash
+
+        salt '*' net.cli textfsm_parse=True textfsm_path=salt://textfsm/
+
     Example output:
 
     .. code-block:: python
 
         {
-            u'show version and haiku':  u'Hostname: re0.edge01.arn01
+            'show version and haiku':  'Hostname: re0.edge01.arn01
                                           Model: mx480
                                           Junos: 13.3R6.5
                                             Help me, Obi-Wan
                                             I just saw Episode Two
                                             You're my only hope
                                          ',
-            u'show chassis fan' :   u'Item                      Status   RPM     Measurement
+            'show chassis fan' :   'Item                      Status   RPM     Measurement
                                       Top Rear Fan              OK       3840    Spinning at intermediate-speed
                                       Bottom Rear Fan           OK       3840    Spinning at intermediate-speed
                                       Top Middle Fan            OK       3900    Spinning at intermediate-speed
@@ -403,9 +526,31 @@ def cli(*commands, **kwargs):  # pylint: disable=unused-argument
                                       Bottom Front Fan          OK       3840    Spinning at intermediate-speed
                                      '
         }
-    '''
 
-    return salt.utils.napalm.call(
+    Example output with TextFSM parsing:
+
+    .. code-block:: json
+
+        {
+          "comment": "",
+          "result": true,
+          "out": {
+            "sh ver": [
+              {
+                "kernel": "9.1S3.5",
+                "documentation": "9.1S3.5",
+                "boot": "9.1S3.5",
+                "crypto": "9.1S3.5",
+                "chassis": "",
+                "routing": "9.1S3.5",
+                "base": "9.1S3.5",
+                "model": "mx960"
+              }
+            ]
+          }
+        }
+    '''
+    raw_cli_outputs = salt.utils.napalm.call(
         napalm_device,  # pylint: disable=undefined-variable
         'cli',
         **{
@@ -414,9 +559,112 @@ def cli(*commands, **kwargs):  # pylint: disable=unused-argument
     )
     # thus we can display the output as is
     # in case of errors, they'll be catched in the proxy
+    if not raw_cli_outputs['result']:
+        # Error -> dispaly the output as-is.
+        return raw_cli_outputs
+    textfsm_parse = kwargs.get('textfsm_parse') or __opts__.get('napalm_cli_textfsm_parse') or\
+                    __pillar__.get('napalm_cli_textfsm_parse', False)
+    if not textfsm_parse:
+        # No TextFSM parsing required, return raw commands.
+        log.debug('No TextFSM parsing requested.')
+        return raw_cli_outputs
+    if 'textfsm.extract' not in __salt__ or 'textfsm.index' not in __salt__:
+        raw_cli_outputs['comment'] += 'Unable to process: is TextFSM installed?'
+        log.error(raw_cli_outputs['comment'])
+        return raw_cli_outputs
+    textfsm_template = kwargs.get('textfsm_template')
+    log.debug('textfsm_template: %s', textfsm_template)
+    textfsm_path = kwargs.get('textfsm_path') or __opts__.get('textfsm_path') or\
+                   __pillar__.get('textfsm_path')
+    log.debug('textfsm_path: %s', textfsm_path)
+    textfsm_template_dict = kwargs.get('textfsm_template_dict') or __opts__.get('napalm_cli_textfsm_template_dict') or\
+                            __pillar__.get('napalm_cli_textfsm_template_dict', {})
+    log.debug('TextFSM command-template mapping: %s', textfsm_template_dict)
+    index_file = kwargs.get('index_file') or __opts__.get('textfsm_index_file') or\
+                 __pillar__.get('textfsm_index_file')
+    log.debug('index_file: %s', index_file)
+    platform_grain_name = kwargs.get('platform_grain_name') or __opts__.get('textfsm_platform_grain') or\
+                          __pillar__.get('textfsm_platform_grain', 'os')
+    log.debug('platform_grain_name: %s', platform_grain_name)
+    platform_column_name = kwargs.get('platform_column_name') or __opts__.get('textfsm_platform_column_name') or\
+                           __pillar__.get('textfsm_platform_column_name', 'Platform')
+    log.debug('platform_column_name: %s', platform_column_name)
+    saltenv = kwargs.get('saltenv', 'base')
+    include_empty = kwargs.get('include_empty', False)
+    include_pat = kwargs.get('include_pat')
+    exclude_pat = kwargs.get('exclude_pat')
+    processed_cli_outputs = {
+        'comment': raw_cli_outputs.get('comment', ''),
+        'result': raw_cli_outputs['result'],
+        'out': {}
+    }
+    log.debug('Starting to analyse the raw outputs')
+    for command in list(commands):
+        command_output = raw_cli_outputs['out'][command]
+        log.debug('Output from command: %s', command)
+        log.debug(command_output)
+        processed_command_output = None
+        if textfsm_path:
+            log.debug('Using the templates under %s', textfsm_path)
+            processed_cli_output = __salt__['textfsm.index'](command,
+                                                             platform_grain_name=platform_grain_name,
+                                                             platform_column_name=platform_column_name,
+                                                             output=command_output.strip(),
+                                                             textfsm_path=textfsm_path,
+                                                             saltenv=saltenv,
+                                                             include_empty=include_empty,
+                                                             include_pat=include_pat,
+                                                             exclude_pat=exclude_pat)
+            log.debug('Processed CLI output:')
+            log.debug(processed_cli_output)
+            if not processed_cli_output['result']:
+                log.debug('Apparently this didnt work, returnin the raw output')
+                processed_command_output = command_output
+                processed_cli_outputs['comment'] += '\nUnable to process the output from {0}: {1}.'.format(command,
+                    processed_cli_output['comment'])
+                log.error(processed_cli_outputs['comment'])
+            elif processed_cli_output['out']:
+                log.debug('All good, %s has a nice output!', command)
+                processed_command_output = processed_cli_output['out']
+            else:
+                comment = '''\nProcessing "{}" didn't fail, but didn't return anything either. Dumping raw.'''.format(
+                    command)
+                processed_cli_outputs['comment'] += comment
+                log.error(comment)
+                processed_command_output = command_output
+        elif textfsm_template or command in textfsm_template_dict:
+            if command in textfsm_template_dict:
+                textfsm_template = textfsm_template_dict[command]
+            log.debug('Using %s to process the command: %s', textfsm_template, command)
+            processed_cli_output = __salt__['textfsm.extract'](textfsm_template,
+                                                               raw_text=command_output,
+                                                               saltenv=saltenv)
+            log.debug('Processed CLI output:')
+            log.debug(processed_cli_output)
+            if not processed_cli_output['result']:
+                log.debug('Apparently this didnt work, returning '
+                          'the raw output')
+                processed_command_output = command_output
+                processed_cli_outputs['comment'] += '\nUnable to process the output from {0}: {1}'.format(command,
+                    processed_cli_output['comment'])
+                log.error(processed_cli_outputs['comment'])
+            elif processed_cli_output['out']:
+                log.debug('All good, %s has a nice output!', command)
+                processed_command_output = processed_cli_output['out']
+            else:
+                log.debug('Processing %s didnt fail, but didnt return'
+                          ' anything either. Dumping raw.', command)
+                processed_command_output = command_output
+        else:
+            log.error('No TextFSM template specified, or no TextFSM path defined')
+            processed_command_output = command_output
+            processed_cli_outputs['comment'] += '\nUnable to process the output from {}.'.format(command)
+        processed_cli_outputs['out'][command] = processed_command_output
+    processed_cli_outputs['comment'] = processed_cli_outputs['comment'].strip()
+    return processed_cli_outputs
 
 
-@proxy_napalm_wrap
+@salt.utils.napalm.proxy_napalm_wrap
 def traceroute(destination, source=None, ttl=None, timeout=None, vrf=None, **kwargs):  # pylint: disable=unused-argument
 
     '''
@@ -461,7 +709,7 @@ def traceroute(destination, source=None, ttl=None, timeout=None, vrf=None, **kwa
     )
 
 
-@proxy_napalm_wrap
+@salt.utils.napalm.proxy_napalm_wrap
 def ping(destination, source=None, ttl=None, timeout=None, size=None, count=None, vrf=None, **kwargs):  # pylint: disable=unused-argument
 
     '''
@@ -514,7 +762,7 @@ def ping(destination, source=None, ttl=None, timeout=None, size=None, count=None
     )
 
 
-@proxy_napalm_wrap
+@salt.utils.napalm.proxy_napalm_wrap
 def arp(interface='', ipaddr='', macaddr='', **kwargs):  # pylint: disable=unused-argument
 
     '''
@@ -580,18 +828,18 @@ def arp(interface='', ipaddr='', macaddr='', **kwargs):  # pylint: disable=unuse
     return proxy_output
 
 
-@proxy_napalm_wrap
+@salt.utils.napalm.proxy_napalm_wrap
 def ipaddrs(**kwargs):  # pylint: disable=unused-argument
 
     '''
     Returns IP addresses configured on the device.
 
-
-    :return:   A dictionary with the IPv4 and IPv6 addresses of the interfaces.\
-    Returns all configured IP addresses on all interfaces as a dictionary of dictionaries.\
-    Keys of the main dictionary represent the name of the interface.\
-    Values of the main dictionary represent are dictionaries that may consist of two keys\
-    'ipv4' and 'ipv6' (one, both or none) which are themselvs dictionaries with the IP addresses as keys.\
+    :return:   A dictionary with the IPv4 and IPv6 addresses of the interfaces.
+        Returns all configured IP addresses on all interfaces as a dictionary
+        of dictionaries.  Keys of the main dictionary represent the name of the
+        interface.  Values of the main dictionary represent are dictionaries
+        that may consist of two keys 'ipv4' and 'ipv6' (one, both or none)
+        which are themselvs dictionaries with the IP addresses as keys.
 
     CLI Example:
 
@@ -604,28 +852,28 @@ def ipaddrs(**kwargs):  # pylint: disable=unused-argument
     .. code-block:: python
 
         {
-            u'FastEthernet8': {
-                u'ipv4': {
-                    u'10.66.43.169': {
+            'FastEthernet8': {
+                'ipv4': {
+                    '10.66.43.169': {
                         'prefix_length': 22
                     }
                 }
             },
-            u'Loopback555': {
-                u'ipv4': {
-                    u'192.168.1.1': {
+            'Loopback555': {
+                'ipv4': {
+                    '192.168.1.1': {
                         'prefix_length': 24
                     }
                 },
-                u'ipv6': {
-                    u'1::1': {
+                'ipv6': {
+                    '1::1': {
                         'prefix_length': 64
                     },
-                    u'2001:DB8:1::1': {
+                    '2001:DB8:1::1': {
                         'prefix_length': 64
                     },
-                    u'FE80::3': {
-                        'prefix_length': u'N/A'
+                    'FE80::3': {
+                        'prefix_length': 'N/A'
                     }
                 }
             }
@@ -640,14 +888,14 @@ def ipaddrs(**kwargs):  # pylint: disable=unused-argument
     )
 
 
-@proxy_napalm_wrap
+@salt.utils.napalm.proxy_napalm_wrap
 def interfaces(**kwargs):  # pylint: disable=unused-argument
 
     '''
     Returns details of the interfaces on the device.
 
-    :return: Returns a dictionary of dictionaries. \
-    The keys for the first dictionary will be the interfaces in the devices.
+    :return: Returns a dictionary of dictionaries. The keys for the first
+        dictionary will be the interfaces in the devices.
 
     CLI Example:
 
@@ -660,21 +908,21 @@ def interfaces(**kwargs):  # pylint: disable=unused-argument
     .. code-block:: python
 
         {
-            u'Management1': {
+            'Management1': {
                 'is_up': False,
                 'is_enabled': False,
-                'description': u'',
+                'description': '',
                 'last_flapped': -1,
                 'speed': 1000,
-                'mac_address': u'dead:beef:dead',
+                'mac_address': 'dead:beef:dead',
             },
-            u'Ethernet1':{
+            'Ethernet1':{
                 'is_up': True,
                 'is_enabled': True,
-                'description': u'foo',
+                'description': 'foo',
                 'last_flapped': 1429978575.1554043,
                 'speed': 1000,
-                'mac_address': u'beef:dead:beef',
+                'mac_address': 'beef:dead:beef',
             }
         }
     '''
@@ -687,15 +935,16 @@ def interfaces(**kwargs):  # pylint: disable=unused-argument
     )
 
 
-@proxy_napalm_wrap
+@salt.utils.napalm.proxy_napalm_wrap
 def lldp(interface='', **kwargs):  # pylint: disable=unused-argument
 
     '''
     Returns a detailed view of the LLDP neighbors.
 
     :param interface: interface name to filter on
-    :return:          A dictionary with the LLDL neighbors.\
-    The keys are the interfaces with LLDP activated on.
+
+    :return:          A dictionary with the LLDL neighbors. The keys are the
+        interfaces with LLDP activated on.
 
     CLI Example:
 
@@ -711,17 +960,17 @@ def lldp(interface='', **kwargs):  # pylint: disable=unused-argument
         {
             'TenGigE0/0/0/8': [
                 {
-                    'parent_interface': u'Bundle-Ether8',
-                    'interface_description': u'TenGigE0/0/0/8',
-                    'remote_chassis_id': u'8c60.4f69.e96c',
-                    'remote_system_name': u'switch',
-                    'remote_port': u'Eth2/2/1',
-                    'remote_port_description': u'Ethernet2/2/1',
-                    'remote_system_description': u'Cisco Nexus Operating System (NX-OS) Software 7.1(0)N1(1a)
+                    'parent_interface': 'Bundle-Ether8',
+                    'interface_description': 'TenGigE0/0/0/8',
+                    'remote_chassis_id': '8c60.4f69.e96c',
+                    'remote_system_name': 'switch',
+                    'remote_port': 'Eth2/2/1',
+                    'remote_port_description': 'Ethernet2/2/1',
+                    'remote_system_description': 'Cisco Nexus Operating System (NX-OS) Software 7.1(0)N1(1a)
                           TAC support: http://www.cisco.com/tac
                           Copyright (c) 2002-2015, Cisco Systems, Inc. All rights reserved.',
-                    'remote_system_capab': u'B, R',
-                    'remote_system_enable_capab': u'B'
+                    'remote_system_capab': 'B, R',
+                    'remote_system_enable_capab': 'B'
                 }
             ]
         }
@@ -749,7 +998,7 @@ def lldp(interface='', **kwargs):  # pylint: disable=unused-argument
     return proxy_output
 
 
-@proxy_napalm_wrap
+@salt.utils.napalm.proxy_napalm_wrap
 def mac(address='', interface='', vlan=0, **kwargs):  # pylint: disable=unused-argument
 
     '''
@@ -822,24 +1071,23 @@ def mac(address='', interface='', vlan=0, **kwargs):  # pylint: disable=unused-a
     return proxy_output
 
 
-@proxy_napalm_wrap
+@salt.utils.napalm.proxy_napalm_wrap
 def config(source=None, **kwargs):  # pylint: disable=unused-argument
     '''
     .. versionadded:: 2017.7.0
 
-    Return the whole configuration of the network device.
-    By default, it will return all possible configuration
-    sources supported by the network device.
+    Return the whole configuration of the network device. By default, it will
+    return all possible configuration sources supported by the network device.
     At most, there will be:
 
     - running config
     - startup config
     - candidate config
 
-    To return only one of the configurations, you can use
-    the ``source`` argument.
+    To return only one of the configurations, you can use the ``source``
+    argument.
 
-    source (optional)
+    source
         Which configuration type you want to display, default is all of them.
 
         Options:
@@ -853,10 +1101,10 @@ def config(source=None, **kwargs):  # pylint: disable=unused-argument
 
         - running (string): Representation of the native running configuration.
         - candidate (string): Representation of the native candidate configuration.
-            If the device doesnt differentiate between running and startup
+            If the device doesn't differentiate between running and startup
             configuration this will an empty string.
         - startup (string): Representation of the native startup configuration.
-            If the device doesnt differentiate between running and startup
+            If the device doesn't differentiate between running and startup
             configuration this will an empty string.
 
     CLI Example:
@@ -875,7 +1123,7 @@ def config(source=None, **kwargs):  # pylint: disable=unused-argument
     )
 
 
-@proxy_napalm_wrap
+@salt.utils.napalm.proxy_napalm_wrap
 def optics(**kwargs):  # pylint: disable=unused-argument
     '''
     .. versionadded:: 2017.7.0
@@ -925,7 +1173,7 @@ def optics(**kwargs):  # pylint: disable=unused-argument
 # ----- Configuration specific functions ------------------------------------------------------------------------------>
 
 
-@proxy_napalm_wrap
+@salt.utils.napalm.proxy_napalm_wrap
 def load_config(filename=None,
                 text=None,
                 test=False,
@@ -959,7 +1207,7 @@ def load_config(filename=None,
         - ``s3://``
         - ``swift://``
 
-        .. versionchanged:: 2017.7.3
+        .. versionchanged:: 2018.3.0
 
     text
         String containing the desired configuration.
@@ -986,7 +1234,7 @@ def load_config(filename=None,
     saltenv: ``base``
         Specifies the Salt environment name.
 
-        .. versionadded:: 2017.7.3
+        .. versionadded:: 2018.3.0
 
     :return: a dictionary having the following keys:
 
@@ -1010,6 +1258,7 @@ def load_config(filename=None,
     Example output:
 
     .. code-block:: python
+
         {
             'comment': 'Configuration discarded.',
             'already_configured': False,
@@ -1050,7 +1299,11 @@ def load_config(filename=None,
     )
     loaded_config = None
     if debug:
-        loaded_config = text
+        if filename:
+            with salt.utils.files.fopen(filename) as rfh:
+                loaded_config = salt.utils.stringutils.to_unicode(rfh.read())
+        else:
+            loaded_config = text
     return _config_logic(napalm_device,  # pylint: disable=undefined-variable
                          _loaded,
                          test=test,
@@ -1058,7 +1311,7 @@ def load_config(filename=None,
                          loaded_config=loaded_config)
 
 
-@proxy_napalm_wrap
+@salt.utils.napalm.proxy_napalm_wrap
 def load_template(template_name,
                   template_source=None,
                   template_path=None,
@@ -1067,6 +1320,7 @@ def load_template(template_name,
                   template_user='root',
                   template_group='root',
                   template_mode='755',
+                  template_attrs='--------------e----',
                   saltenv=None,
                   template_engine='jinja',
                   skip_verify=False,
@@ -1160,11 +1414,16 @@ def load_template(template_name,
 
         .. versionadded:: 2016.11.2
 
-    template_user: 755
+    template_mode: 755
         Permissions of file.
 
         .. versionadded:: 2016.11.2
 
+    template_attrs: "--------------e----"
+        attributes of file. (see `man lsattr`)
+
+        .. versionadded:: 2018.3.0
+
     saltenv: base
         Specifies the template environment.
         This will influence the relative imports inside the templates.
@@ -1214,26 +1473,26 @@ def load_template(template_name,
 
         .. versionadded:: 2016.11.2
 
-    **template_vars
+    template_vars
         Dictionary with the arguments/context to be used when the template is rendered.
 
         .. note::
 
-            Do not explicitly specify this argument.
-            This represents any other variable that will be sent
-            to the template rendering system.
+            Do not explicitly specify this argument. This represents any other
+            variable that will be sent to the template rendering system.
             Please see the examples below!
 
     :return: a dictionary having the following keys:
 
-    * result (bool): if the config was applied successfully. It is ``False`` only in case of failure. In case \
-    there are no changes to be applied and successfully performs all operations it is still ``True`` and so will be \
-    the ``already_configured`` flag (example below)
-    * comment (str): a message for the user
-    * already_configured (bool): flag to check if there were no changes applied
-    * loaded_config (str): the configuration loaded on the device, after rendering the template. Requires ``debug`` \
-    to be set as ``True``
-    * diff (str): returns the config changes applied
+    - result (bool): if the config was applied successfully. It is ``False``
+      only in case of failure. In case there are no changes to be applied and
+      successfully performs all operations it is still ``True`` and so will be
+      the ``already_configured`` flag (example below)
+    - comment (str): a message for the user
+    - already_configured (bool): flag to check if there were no changes applied
+    - loaded_config (str): the configuration loaded on the device, after
+      rendering the template. Requires ``debug`` to be set as ``True``
+    - diff (str): returns the config changes applied
 
     The template can use variables from the ``grains``, ``pillar`` or ``opts``, for example:
 
@@ -1366,7 +1625,7 @@ def load_template(template_name,
                     # use the custom template path
                     saltenv = template_path if not salt_render else 'base'
             elif salt_render and not saltenv:
-                # if saltenv not overrided and path specified as salt:// or http:// etc.
+                # if saltenv not overridden and path specified as salt:// or http:// etc.
                 # will use the default environment, from the base
                 saltenv = template_path if template_path else 'base'
             if not saltenv:
@@ -1382,6 +1641,7 @@ def load_template(template_name,
                                                     user=template_user,
                                                     group=template_group,
                                                     mode=template_mode,
+                                                    attrs=template_attrs,
                                                     template=template_engine,
                                                     context=template_vars,
                                                     defaults=defaults,
@@ -1403,8 +1663,8 @@ def load_template(template_name,
                     _loaded['result'] = False
                     _loaded['comment'] = 'Error while rendering the template.'
                     return _loaded
-                with salt.utils.fopen(_temp_tpl_file) as rfh:
-                    _rendered = rfh.read()
+                with salt.utils.files.fopen(_temp_tpl_file) as rfh:
+                    _rendered = salt.utils.stringutils.to_unicode(rfh.read())
                 __salt__['file.remove'](_temp_tpl_file)
             else:
                 return _loaded  # exit
@@ -1454,7 +1714,7 @@ def load_template(template_name,
             # after running the other features:
             # compare_config, discard / commit
             # which have to be over the same session
-            # so we'll set the CLOSE global explicitely as False
+            # so we'll set the CLOSE global explicitly as False
             napalm_device['CLOSE'] = False  # pylint: disable=undefined-variable
         _loaded = salt.utils.napalm.call(
             napalm_device,  # pylint: disable=undefined-variable
@@ -1468,7 +1728,7 @@ def load_template(template_name,
                          loaded_config=loaded_config)
 
 
-@proxy_napalm_wrap
+@salt.utils.napalm.proxy_napalm_wrap
 def commit(inherit_napalm_device=None, **kwargs):  # pylint: disable=unused-argument
 
     '''
@@ -1488,7 +1748,7 @@ def commit(inherit_napalm_device=None, **kwargs):  # pylint: disable=unused-argu
     )
 
 
-@proxy_napalm_wrap
+@salt.utils.napalm.proxy_napalm_wrap
 def discard_config(inherit_napalm_device=None, **kwargs):  # pylint: disable=unused-argument
 
     """
@@ -1508,7 +1768,7 @@ def discard_config(inherit_napalm_device=None, **kwargs):  # pylint: disable=unu
     )
 
 
-@proxy_napalm_wrap
+@salt.utils.napalm.proxy_napalm_wrap
 def compare_config(inherit_napalm_device=None, **kwargs):  # pylint: disable=unused-argument
 
     '''
@@ -1528,7 +1788,7 @@ def compare_config(inherit_napalm_device=None, **kwargs):  # pylint: disable=unu
     )
 
 
-@proxy_napalm_wrap
+@salt.utils.napalm.proxy_napalm_wrap
 def rollback(inherit_napalm_device=None, **kwargs):  # pylint: disable=unused-argument
 
     '''
@@ -1548,7 +1808,7 @@ def rollback(inherit_napalm_device=None, **kwargs):  # pylint: disable=unused-ar
     )
 
 
-@proxy_napalm_wrap
+@salt.utils.napalm.proxy_napalm_wrap
 def config_changed(inherit_napalm_device=None, **kwargs):  # pylint: disable=unused-argument
 
     '''
@@ -1579,7 +1839,7 @@ def config_changed(inherit_napalm_device=None, **kwargs):  # pylint: disable=unu
     return is_config_changed, reason
 
 
-@proxy_napalm_wrap
+@salt.utils.napalm.proxy_napalm_wrap
 def config_control(inherit_napalm_device=None, **kwargs):  # pylint: disable=unused-argument
 
     '''
