@@ -12,17 +12,19 @@ Execute calls on selinux
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 import os
 import re
 
 # Import salt libs
-import salt.utils
+import salt.utils.files
+import salt.utils.path
+import salt.utils.stringutils
 import salt.utils.decorators as decorators
 from salt.exceptions import CommandExecutionError, SaltInvocationError
 
 # Import 3rd-party libs
-import salt.ext.six as six
+from salt.ext import six
 
 
 _SELINUX_FILETYPES = {
@@ -46,7 +48,7 @@ def __virtual__():
     # Iterate over all of the commands this module uses and make sure
     # each of them are available in the standard PATH to prevent breakage
     for cmd in required_cmds:
-        if not salt.utils.which(cmd):
+        if not salt.utils.path.which(cmd):
             return (False, cmd + ' is not in the path')
     # SELinux only makes sense on Linux *obviously*
     if __grains__['kernel'] == 'Linux':
@@ -94,8 +96,8 @@ def getenforce():
         return 'Disabled'
     try:
         enforce = os.path.join(_selinux_fs_path, 'enforce')
-        with salt.utils.fopen(enforce, 'r') as _fp:
-            if _fp.readline().strip() == '0':
+        with salt.utils.files.fopen(enforce, 'r') as _fp:
+            if salt.utils.stringutils.to_unicode(_fp.readline()).strip() == '0':
                 return 'Permissive'
             else:
                 return 'Enforcing'
@@ -115,8 +117,9 @@ def getconfig():
     '''
     try:
         config = '/etc/selinux/config'
-        with salt.utils.fopen(config, 'r') as _fp:
+        with salt.utils.files.fopen(config, 'r') as _fp:
             for line in _fp:
+                line = salt.utils.stringutils.to_unicode(line)
                 if line.strip().startswith('SELINUX='):
                     return line.split('=')[1].capitalize().strip()
     except (IOError, OSError, AttributeError):
@@ -158,26 +161,26 @@ def setenforce(mode):
     if getenforce() != 'Disabled':
         enforce = os.path.join(selinux_fs_path(), 'enforce')
         try:
-            with salt.utils.fopen(enforce, 'w') as _fp:
-                _fp.write(mode)
+            with salt.utils.files.fopen(enforce, 'w') as _fp:
+                _fp.write(salt.utils.stringutils.to_str(mode))
         except (IOError, OSError) as exc:
             msg = 'Could not write SELinux enforce file: {0}'
-            raise CommandExecutionError(msg.format(str(exc)))
+            raise CommandExecutionError(msg.format(exc))
 
     config = '/etc/selinux/config'
     try:
-        with salt.utils.fopen(config, 'r') as _cf:
+        with salt.utils.files.fopen(config, 'r') as _cf:
             conf = _cf.read()
         try:
-            with salt.utils.fopen(config, 'w') as _cf:
+            with salt.utils.files.fopen(config, 'w') as _cf:
                 conf = re.sub(r"\nSELINUX=.*\n", "\nSELINUX=" + modestring + "\n", conf)
-                _cf.write(conf)
+                _cf.write(salt.utils.stringutils.to_str(conf))
         except (IOError, OSError) as exc:
             msg = 'Could not write SELinux config file: {0}'
-            raise CommandExecutionError(msg.format(str(exc)))
+            raise CommandExecutionError(msg.format(exc))
     except (IOError, OSError) as exc:
         msg = 'Could not read SELinux config file: {0}'
-        raise CommandExecutionError(msg.format(str(exc)))
+        raise CommandExecutionError(msg.format(exc))
 
     return getenforce()
 
@@ -453,7 +456,7 @@ def fcontext_get_policy(name, filetype=None, sel_type=None, sel_user=None, sel_l
     '''
     if filetype:
         _validate_filetype(filetype)
-    re_spacer = '[ ]{2,}'
+    re_spacer = '[ ]+'
     cmd_kwargs = {'spacer': re_spacer,
                   'filespec': re.escape(name),
                   'sel_user': sel_user or '[^:]+',
@@ -466,11 +469,14 @@ def fcontext_get_policy(name, filetype=None, sel_type=None, sel_user=None, sel_l
     current_entry_text = __salt__['cmd.shell'](cmd, ignore_retcode=True)
     if current_entry_text == '':
         return None
-    ret = {}
-    current_entry_list = re.split(re_spacer, current_entry_text)
-    ret['filespec'] = current_entry_list[0]
-    ret['filetype'] = current_entry_list[1]
-    ret.update(_context_string_to_dict(current_entry_list[2]))
+
+    parts = re.match(r'^({filespec}) +([a-z ]+) (.*)$'.format(**{'filespec': re.escape(name)}), current_entry_text)
+    ret = {
+        'filespec': parts.group(1).strip(),
+        'filetype': parts.group(2).strip(),
+    }
+    ret.update(_context_string_to_dict(parts.group(3).strip()))
+
     return ret
 
 
@@ -514,7 +520,9 @@ def fcontext_add_or_delete_policy(action, name, filetype=None, sel_type=None, se
     if action not in ['add', 'delete']:
         raise SaltInvocationError('Actions supported are "add" and "delete", not "{0}".'.format(action))
     cmd = 'semanage fcontext --{0}'.format(action)
-    if filetype is not None:
+    # "semanage --ftype a" isn't valid on Centos 6,
+    # don't pass --ftype since "a" is the default filetype.
+    if filetype is not None and filetype != 'a':
         _validate_filetype(filetype)
         cmd += ' --ftype {0}'.format(filetype)
     if sel_type is not None:
@@ -556,7 +564,7 @@ def fcontext_apply_policy(name, recursive=False):
     .. versionadded:: 2017.7.0
 
     Applies SElinux policies to filespec using `restorecon [-R]
-    filespec`. Returns dict with changes if succesful, the output of
+    filespec`. Returns dict with changes if successful, the output of
     the restorecon command otherwise.
 
     name
