@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 '''
-    :codeauthor: :email:`Pedro Algarvio (pedro@algarvio.me)`
+    :codeauthor: Pedro Algarvio (pedro@algarvio.me)
 
     tests.conftest
     ~~~~~~~~~~~~~~
@@ -46,21 +46,26 @@ from _pytest.terminal import TerminalReporter
 
 # Import 3rd-party libs
 import psutil
-import salt.ext.six as six
+from salt.ext import six
 
 # Import salt libs
-import salt.utils
+import salt.utils.files
+import salt.utils.path
 import salt.log.setup
 from salt.utils.odict import OrderedDict
 
 # Define the pytest plugins we rely on
-pytest_plugins = ['pytest_catchlog', 'tempdir', 'helpers_namespace']  # pylint: disable=invalid-name
+pytest_plugins = ['tempdir', 'helpers_namespace']  # pylint: disable=invalid-name
 
 # Define where not to collect tests from
 collect_ignore = ['setup.py']
 
 log = logging.getLogger('salt.testsuite')
 
+# Reset logging root handlers
+for handler in logging.root.handlers:
+    logging.root.removeHandler(handler)
+
 
 def pytest_tempdir_basename():
     '''
@@ -98,6 +103,14 @@ def pytest_addoption(parser):
              'SSH server on your machine. In certain environments, this '
              'may be insecure! Default: False'
     )
+    test_selection_group.addoption(
+        '--proxy',
+        '--proxy-tests',
+        dest='proxy',
+        action='store_true',
+        default=False,
+        help='Run proxy tests'
+    )
     test_selection_group.addoption(
         '--run-destructive',
         action='store_true',
@@ -196,25 +209,6 @@ def pytest_configure(config):
     called after command line options have been parsed
     and all plugins and initial conftest files been loaded.
     '''
-    # Configure the console logger based on the catch_log settings.
-    # Most importantly, shutdown Salt's null, store and temporary logging queue handlers
-    catch_log = config.pluginmanager.getplugin('_catch_log')
-    cli_logging_handler = catch_log.log_cli_handler
-    # Add the pytest_catchlog CLI log handler to the logging root
-    logging.root.addHandler(cli_logging_handler)
-    cli_level = cli_logging_handler.level
-    cli_level = config._catchlog_log_cli_level
-    cli_format = cli_logging_handler.formatter._fmt
-    cli_date_format = cli_logging_handler.formatter.datefmt
-    # Setup the console logger which shuts down the null and the temporary queue handlers
-    salt.log.setup_console_logger(
-        log_level=salt.log.setup.LOG_VALUES_TO_LEVELS.get(cli_level, 'error'),
-        log_format=cli_format,
-        date_format=cli_date_format
-    )
-    # Disable the store logging queue handler
-    salt.log.setup.setup_extended_logging({'extension_modules': ''})
-
     config.addinivalue_line('norecursedirs', os.path.join(CODE_DIR, 'templates'))
     config.addinivalue_line(
         'markers',
@@ -260,11 +254,13 @@ def pytest_runtest_setup(item):
     if destructive_tests_marker is not None:
         if item.config.getoption('--run-destructive') is False:
             pytest.skip('Destructive tests are disabled')
+    os.environ['DESTRUCTIVE_TESTS'] = six.text_type(item.config.getoption('--run-destructive'))
 
     expensive_tests_marker = item.get_marker('expensive_test')
     if expensive_tests_marker is not None:
         if item.config.getoption('--run-expensive') is False:
             pytest.skip('Expensive tests are disabled')
+    os.environ['EXPENSIVE_TESTS'] = six.text_type(item.config.getoption('--run-expensive'))
 
     skip_if_not_root_marker = item.get_marker('skip_if_not_root')
     if skip_if_not_root_marker is not None:
@@ -281,14 +277,14 @@ def pytest_runtest_setup(item):
         message = skip_if_binaries_missing_marker.kwargs.get('message', None)
         if check_all:
             for binary in binaries:
-                if salt.utils.which(binary) is None:
+                if salt.utils.path.which(binary) is None:
                     pytest.skip(
                         '{0}The "{1}" binary was not found'.format(
                             message and '{0}. '.format(message) or '',
                             binary
                         )
                     )
-        elif salt.utils.which_bin(binaries) is None:
+        elif salt.utils.path.which_bin(binaries) is None:
             pytest.skip(
                 '{0}None of the following binaries was found: {1}'.format(
                     message and '{0}. '.format(message) or '',
@@ -592,7 +588,7 @@ def cli_bin_dir(tempdir,
         if not os.path.isfile(script_path):
             log.info('Generating {0}'.format(script_path))
 
-            with salt.utils.fopen(script_path, 'w') as sfh:
+            with salt.utils.files.fopen(script_path, 'w') as sfh:
                 script_template = script_templates.get(original_script_name, None)
                 if script_template is None:
                     script_template = script_templates.get('common', None)
@@ -655,7 +651,8 @@ def test_daemon(request):
               ('sysinfo', request.config.getoption('--sysinfo')),
               ('no_colors', request.config.getoption('--no-colors')),
               ('output_columns', request.config.getoption('--output-columns')),
-              ('ssh', request.config.getoption('--ssh')))
+              ('ssh', request.config.getoption('--ssh')),
+              ('proxy', request.config.getoption('--proxy')))
     options = namedtuple('options', [n for n, v in values])(*[v for n, v in values])
     fake_parser = namedtuple('parser', 'options')(options)
 
