@@ -1,17 +1,17 @@
 # -*- coding: utf-8 -*-
 
 # Import Python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import Salt Testing Libs
 from tests.support.case import SSHCase
 from tests.support.unit import skipIf
 
 # Import Salt Libs
-import salt.utils
+import salt.utils.platform
 
 
-@skipIf(salt.utils.is_windows(), 'salt-ssh not available on Windows')
+@skipIf(salt.utils.platform.is_windows(), 'salt-ssh not available on Windows')
 class SSHPillarTest(SSHCase):
     '''
     testing pillar with salt-ssh
