@@ -4,11 +4,21 @@ tests for host state
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 
 # Import Salt Testing libs
 from tests.support.case import ModuleCase
 
+# Import Salt libs
+import salt.utils.platform
+
+# Import 3rd-Party libs
+HAS_LSB_RELEASE = True
+try:
+    import lsb_release
+except ImportError:
+    HAS_LSB_RELEASE = False
+
 
 class CompileTest(ModuleCase):
     '''
@@ -27,6 +37,11 @@ class CompileTest(ModuleCase):
         Test when we have an error in a execution module
         called by jinja
         '''
+        if salt.utils.platform.is_linux() and HAS_LSB_RELEASE:
+            release = lsb_release.get_distro_information()
+            if release.get('ID') == 'Debian' and int(release.get('RELEASE', '0')[0]) < 9:
+                self.skipTest('This test is flaky on Debian 8. Skipping.')
+
         ret = self.run_function('state.sls', ['issue-10010'])
         self.assertTrue(
             ', in jinja_error' in ret[0].strip())
