@@ -4,7 +4,7 @@ tests for host state
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import os
 import shutil
 
@@ -14,7 +14,8 @@ from tests.support.paths import FILES, TMP
 from tests.support.mixins import SaltReturnAssertsMixin
 
 # Import salt libs
-import salt.utils
+import salt.utils.files
+import salt.utils.stringutils
 
 HFILE = os.path.join(TMP, 'hosts')
 
@@ -41,6 +42,6 @@ class HostTest(ModuleCase, SaltReturnAssertsMixin):
         ip = '10.10.10.10'
         ret = self.run_state('host.present', name=name, ip=ip)
         self.assertSaltTrueReturn(ret)
-        with salt.utils.fopen(HFILE) as fp_:
-            output = fp_.read()
+        with salt.utils.files.fopen(HFILE) as fp_:
+            output = salt.utils.stringutils.to_unicode(fp_.read())
             self.assertIn('{0}\t\t{1}'.format(ip, name), output)
