@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 '''
-    :codeauthor: :email:`Pedro Algarvio (pedro@algarvio.me)`
+    :codeauthor: Pedro Algarvio (pedro@algarvio.me)
 
 
     tests.integration.states.match
@@ -8,7 +8,7 @@
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import os
 
 # Import Salt Testing libs
@@ -17,7 +17,8 @@ from tests.support.paths import FILES
 from tests.support.helpers import skip_if_not_root
 
 # Import salt libs
-import salt.utils
+import salt.utils.files
+import salt.utils.stringutils
 
 STATE_DIR = os.path.join(FILES, 'file', 'base')
 
@@ -34,12 +35,14 @@ class StateMatchTest(ModuleCase):
         top_filename = 'issue-2167-ipcidr-match.sls'
         top_file = os.path.join(STATE_DIR, top_filename)
         try:
-            with salt.utils.fopen(top_file, 'w') as fp_:
+            with salt.utils.files.fopen(top_file, 'w') as fp_:
                 fp_.write(
-                    'base:\n'
-                    '  {0}:\n'
-                    '    - match: ipcidr\n'
-                    '    - test\n'.format(subnets[0])
+                    salt.utils.stringutils.to_str(
+                        'base:\n'
+                        '  {0}:\n'
+                        '    - match: ipcidr\n'
+                        '    - test\n'.format(subnets[0])
+                    )
                 )
             ret = self.run_function('state.top', [top_filename])
             self.assertNotIn(
