@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 '''
-    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
+    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
 '''
 # Import Python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import Salt Testing Libs
 from tests.support.mixins import LoaderModuleMockMixin
@@ -25,7 +25,7 @@ class ChefTestCase(TestCase, LoaderModuleMockMixin):
     Test cases for salt.modules.chef
     '''
     def setup_loader_modules(self):
-        patcher = patch('salt.utils.which', MagicMock(return_value=True))
+        patcher = patch('salt.utils.path.which', MagicMock(return_value=True))
         patcher.start()
         self.addCleanup(patcher.stop)
         return {chef: {'_exec_cmd': MagicMock(return_value={})}}
