@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # Import Python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import Salt Testing libs
 from tests.support.mixins import LoaderModuleMockMixin
@@ -16,7 +16,7 @@ import salt.modules.uwsgi as uwsgi
 class UwsgiTestCase(TestCase, LoaderModuleMockMixin):
 
     def setup_loader_modules(self):
-        patcher = patch('salt.utils.which', Mock(return_value='/usr/bin/uwsgi'))
+        patcher = patch('salt.utils.path.which', Mock(return_value='/usr/bin/uwsgi'))
         patcher.start()
         self.addCleanup(patcher.stop)
         return {uwsgi: {}}
