@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import Salt Testing libs
 from tests.support.mixins import LoaderModuleMockMixin
@@ -27,8 +27,10 @@ fake_pillar_name = 'fake_pillar_name'
 
 def side_effect(group_sel, t):
     if group_sel.find(fake_minion_id) != -1:
-        return [fake_minion_id, ]
-    return ['another_minion_id', ]
+        return {'minions': [fake_minion_id, ],
+                'missing': []}
+    return {'minions': ['another_minion_id', ],
+            'missing': []}
 
 
 class NodegroupsPillarTestCase(TestCase, LoaderModuleMockMixin):
